
const Lang = imports.lang;
const Gio = imports.gi.Gio;
const GLib = imports.gi.GLib;
const Gtk = imports.gi.Gtk;
const Gdk = imports.gi.Gdk;
const GIR = imports.gi.GIRepository;
const GWeather = imports.gi.GWeather;
const GnomeDesktop = imports.gi.GnomeDesktop;
const ExtensionUtils = imports.misc.extensionUtils;
const Me = ExtensionUtils.getCurrentExtension();
const Convenience = Me.imports.convenience;
const Mainloop = imports.mainloop;

let settings, clocksSettings, openWinsCounter, clock;

function init() {
}

const ListStoreColumns = {
  REFERENCE: 0,
  CODE: 1,
  DISPLAY: 2,
  VISIBLE: 3,
  INFEASIBLE: 4,
  LOCAL: 5,
  ORIG_DISPLAY: 6
};

let clocks_inhibitor, codeMap, localValues = {};

function thawCodeMap(code, obj, world, initial) {
  let loc = world.find_by_station_code(code);
  if (loc) {
    obj.loc = loc;
    codeMap[code] = obj;
  }
}

function get_code_map() {
  let [succ, pid, pin, pout, perr] = GLib.spawn_async_with_pipes(
    null, ['gjs-console', Me.dir.get_child('cache.js').get_path()],
    null, GLib.SpawnFlags.SEARCH_PATH, null);
  let channel = GLib.IOChannel.unix_new(pout);
  channel.set_close_on_unref(true);
  return JSON.parse(channel.read_to_end()[1]);
}

function setupChooser(ui, world) {
  if (!codeMap) {
    codeMap = {};
    let codemap = get_code_map();
    let code_;
    for (code_ in codemap) {
      thawCodeMap(code_, codemap[code_], world);
    }
  }

  let entry = ui.get_object('cw-entry');
  let entry_list = [];
  let entry_list_no_sort = true;

  let empty = { i2: [] };

  let filter = ui.get_object('location-filter');
  filter.set_visible_func(function (model, pi) {
    if (!entry_list.length) { return false; }
    let code = model.get_value(pi, 0);
    let v = (codeMap[code]||empty).i2;
    return entry_list.every(function (e) {
      return v.some(function (f) {
	return f.match(e);
      });
    });
  });

  let sorter = ui.get_object('location-sorter');
  sorter.set_default_sort_func(function (model, ai, bi) {
    if (entry_list_no_sort) { return 0; }
    let code_a = model.get_value(ai, 0);
    let code_b = model.get_value(bi, 0);
    let v_ = [ (codeMap[code_a]||empty).i2, (codeMap[code_b]||empty).i2 ];
    let r = entry_list.reduce(function (pv, cv) {
      let mc_ = v_.map(function (e) {
	let mult = e.length;
	return e.map(function (f) {
	  mult -= 1;
	  let m = f.match(cv) || [];
	  return (1+(mult+1)/e.length)*m.length;
	}).reduce(function (x, y) x + y );
      });
      return [0, 1].map(function (i) pv[i] + mc_[i]);
    }, [0, 0]);
    if (r[1] == r[0]) {
      let vas = v_[0].concat(code_a).join(' | ');
      let vbs = v_[1].concat(code_b).join(' | ');
      if (vas > vbs) { return -1; }
      if (vbs > vas) { return 1; }
      return 0;
    }
    return r[1] <= r[0] ? -1 : 1;
  });

  let ls = ui.get_object('location-store');
  let code_;
  for (code_ in codeMap) {
    let e = codeMap[code_];
    ls.insert_with_valuesv(-1, [0,1,2,3,4,5], [
      code_,
      e.d[0] + (e.d2.length && e.d2.length <= 2 ? " (" + e.d2.reverse().join(", ") + ")" : ""),
      e.d[1],
      e.d.slice(2).reverse().join(" / "),
      e.tz_names.join(", "),
      e.tz_offsets.map(function (e) e.slice(3)).join("  ")
    ]);
  }

  let timeout;
  entry.connect('changed', function () {
    if (timeout) { Mainloop.source_remove(timeout); }
    timeout = Mainloop.timeout_add_seconds(1, function () {
      entry_list = entry.text.split(' ').map(function (e) {
	try { return new RegExp(e.replace(/\+/g, '[+ ]'), 'ig'); }
	catch (err) { return null; }
      }).filter(function (e) e && e.source);
      entry_list_no_sort = (entry_list.map(function (e) e.source).join('').length <= 2);
      filter.refilter();
    });
  });
}

const GladeSignalHandlers = new Lang.Class({
  Name: 'GladeSignalHandlers',
  
  _init: function (ui, world) {
    this.ui = ui;
    this.world = world;
  },

  'visible-change': function (object, row) {
    let ls = this.ui.get_object('location-list');
    let [valid, ic] = ls.get_iter_from_string(row);
    if (valid) {
      let hidden = [];
      ls.foreach(function (store, path, iter) {
	let visible = path.to_string() == row ? !object.active : store.get_value(iter, ListStoreColumns.VISIBLE);
	if (!visible) {
	  hidden.push(store.get_value(iter, ListStoreColumns.CODE));
	}
      });
      settings.set_value('hidden', new GLib.Variant('as', hidden));
    }
  },

  'display-edited': function (object, row, new_text) {
    let ls = this.ui.get_object('location-list');
    let [valid, ic] = ls.get_iter_from_string(row);
    if (valid) {
      if (new_text == ls.get_value(ic, ListStoreColumns.DISPLAY)) { return; }
      
      let nameMap = settings.get_value('name-map').deep_unpack();
      let code = ls.get_value(ic, ListStoreColumns.CODE);
      let oname = ls.get_value(ic, ListStoreColumns.ORIG_DISPLAY);
      let [,sfx] = oname.match(/ \((\w+)\)$/);
      if (new_text.match(/^\s*$/)) {
	new_text = oname;
      }
      new_text = new_text.replace(new RegExp(" \\("+sfx+"\\)\\s*$"), "");
      nameMap[code] = new_text;
      new_text = new_text + (sfx ? " (" + sfx + ")" : "");
      ls.set_value(ic, ListStoreColumns.DISPLAY, new_text);
      if (oname != new_text || delete nameMap[code]) {
	let newNameMap = {};
	ls.foreach(function (store, path, iter) {
	  let display = path.to_string() == row ? new_text : store.get_value(iter, ListStoreColumns.DISPLAY);
	  let code = store.get_value(iter, ListStoreColumns.CODE);
	  if (display != store.get_value(iter, ListStoreColumns.ORIG_DISPLAY) && nameMap[code]) {
	    newNameMap[code] = nameMap[code];
	  }
	});
	settings.set_value('name-map', new GLib.Variant('a{ss}', newNameMap));
      }
    }
  },

  'list-remove': function (store, path) {
    if (clocks_inhibitor) { return; }
    let na = this.ui.get_object('num-adj');
    let isMax = na.get_upper() <= na.get_value();
    let sv = clocksSettings.get_value('world-clocks');
    let oldClocks = sv.deep_unpack();
    let clocks = [];

    store.foreach(function (store, path, iter) {
      let representation = store.get_value(iter, ListStoreColumns.REFERENCE);
      let remain = oldClocks.filter(function (cl) {
	return cl.location.get_normal_form().print(true) == representation;
      });
      if (remain.length) {
	clocks.push(remain[0]);
      }
    });
    
    let newUpper = clocks.length + 1;
    na.set_upper(newUpper);
    if (isMax) { na.set_value(newUpper); }

    clocksSettings.set_value('world-clocks', new GLib.Variant('aa{sv}' /* sv.get_type_string() */, clocks));
    if (this.keep_selection) {
      this.ui.get_object('ll-list').get_selection().select_iter(this.keep_selection);
      this.keep_selection = null;
    }
  },

  'list-changed': function (store, path, iter) {
    this.keep_selection = iter;
  },

  'll-list-select': function (selection) {
    let [has, store, iter] = selection.get_selected();
    this.ui.get_object('remove-loc').set_sensitive(has && iter);
  },

  'ad-changed': function (selection) {
    let [has, store, iter] = selection.get_selected();
    this.ui.get_object('ad-ok-button').set_sensitive(
      has && iter);
  },

  'add-loc': function (object) {
    if (!this.addDialog) {
      this.addDialog = this.ui.get_object('add-dialog');
      this.ui.get_object('cw-treeview').get_selection().connect('changed', Lang.bind(this, this['ad-changed']));
      setupChooser(this.ui, this.world);
    }
    this.ui.get_object('ad-ok-button').set_sensitive(false);
    this.ui.get_object('cw-entry').set_text("");
    this.addDialog.show();
  },

  'ad-cancel': function (object) {
    this.addDialog.hide();
  },

  'ad-close': function (object) {
    this.addDialog.hide();
    return true;
  },
  
  'ad-added': function (object) {
    let [has, store, iter] = this.ui.get_object('cw-treeview').get_selection().get_selected();
    let loc;
    if (has && iter) { loc = codeMap[store.get_value(iter, 0)]; }
    if (loc) { loc = loc.loc; }
    if (loc != null) {
      let lbt = new Convenience.LocationBasedTime(loc);
      let isNew = true;
      let ls = this.ui.get_object('location-list');
      ls.foreach(function (store, path, iter) {
	if (store.get_value(iter, ListStoreColumns.CODE) == lbt.code) {
	  isNew = false;
	}
      });
      if (isNew) {
	let isLocal = lbt.tzSameAsLocal();
	let ser = loc.serialize();
	let hidden = settings.get_value('hidden').deep_unpack();
	let turnedOff = !settings.get_boolean('in-calendar');
	let hideLocal = settings.get_boolean('hide-local');
	let nameMap = settings.get_value('name-map').deep_unpack();
	let display = lbt.displayName2;
	if (nameMap[lbt.code]) {
	  let [,sfx] = display.match(/ \((\w+)\)$/);
	  display = nameMap[lbt.code] + (sfx ? " (" + sfx + ")" : "");
	}
	let iter = ls.insert_with_valuesv(-1, [ListStoreColumns.REFERENCE,ListStoreColumns.CODE,ListStoreColumns.ORIG_DISPLAY,ListStoreColumns.DISPLAY,ListStoreColumns.VISIBLE,ListStoreColumns.LOCAL,ListStoreColumns.INFEASIBLE], [
	  ser.get_normal_form().print(true),
	  lbt.code,
	  lbt.displayName2,
	  display,
	  !hidden.filter(function (v) { return v == lbt.code; }).length,
	  isLocal,
	  turnedOff ? true : hideLocal ? isLocal : false
	]);
	let m = this.ui.get_object('ll-list');
	m.get_selection().select_iter(iter);
	m.scroll_to_cell(Gtk.TreePath.new_from_string(this.ui.get_object('location-list').get_string_from_iter(iter)), null, false, 0, 0);
	let clocks = clocksSettings.get_value('world-clocks').deep_unpack();
	clocks.push({ location: ser });
	clocksSettings.set_value('world-clocks', new GLib.Variant('aa{sv}' /* sv.get_type_string() */, clocks));

	let na = this.ui.get_object('num-adj');
	let isMax = na.get_upper() <= na.get_value();
    
	let newUpper = clocks.length + 1;
	na.set_upper(newUpper);
	if (isMax) { na.set_value(newUpper); }
      }
    }
    this.addDialog.hide();
  },

  'remove-loc-sel': function (object) {
    let [has, store, iter] = this.ui.get_object('ll-list').get_selection().get_selected();
    if (has && iter) {
      store.remove(iter);
    }
  },
  
  _connector: function(builder, object, signal, handler) {
    if (this[handler]) {
      object.connect(signal, Lang.bind(this, this[handler]));
    }
  }
});

let position_inhibitor = false;

function buildPrefsWidget() {
  let ui = new Gtk.Builder(), win = new Gtk.Table();
  ui.add_from_file(Me.dir.get_child('prefs.ui').get_path());

  ui.get_object('content-table').reparent(win);

  if (!settings) {
    settings = Convenience.getSettings();
    clocksSettings = Convenience.getClocksSettings();    
    clock = new GnomeDesktop.WallClock();
    openWinsCounter = 0;
  }
  else {
    openWinsCounter = openWinsCounter + 1;
  }

  let customTheme = new Gtk.CssProvider();
  customTheme.load_from_file(Me.dir.get_child('stylesheet.css'));
  Gtk.StyleContext.add_provider_for_screen(win.get_screen(), customTheme, Gtk.STYLE_PROVIDER_PRIORITY_APPLICATION);
  
  settings.bind('hide-local', ui.get_object('local-cb'), 'active', Gio.SettingsBindFlags.DEFAULT);
  settings.bind('in-calendar', ui.get_object('calendar-sw'), 'active', Gio.SettingsBindFlags.DEFAULT);
  settings.bind('num-buttons', ui.get_object('num-adj'), 'value', Gio.SettingsBindFlags.DEFAULT);

  ui.get_object('remove-loc').set_sensitive(false);  
  let world = GWeather.Location.new_world(true);
 
  let worldClocksUpdate = function () {
    if (clocks_inhibitor) { return; }
    clocks_inhibitor = true;
    let na = ui.get_object('num-adj');
    let isMax = na.get_upper() <= na.get_value();
    let clocks = clocksSettings.get_value('world-clocks').deep_unpack();
    
    let newUpper = clocks.length + 1;
    na.set_upper(newUpper);
    if (isMax) { na.set_value(newUpper); }

    let ls = ui.get_object('location-list');
    ls.clear();
    let hidden = settings.get_value('hidden').deep_unpack();
    let nameMap = settings.get_value('name-map').deep_unpack();
    let turnedOff = !settings.get_boolean('in-calendar');
    let hideLocal = settings.get_boolean('hide-local');
    clocks.map(function (cl) {
      let lbt = new Convenience.LocationBasedTime(world.deserialize(cl.location));
      let isLocal = lbt.tzSameAsLocal();
      let display = lbt.displayName2;
      if (nameMap[lbt.code]) {
	let [,sfx] = display.match(/ \((\w+)\)$/);
	display = nameMap[lbt.code] + (sfx ? " (" + sfx + ")" : "");
      }
      
      ls.insert_with_valuesv(-1, [ListStoreColumns.REFERENCE,ListStoreColumns.CODE,ListStoreColumns.ORIG_DISPLAY,ListStoreColumns.DISPLAY,ListStoreColumns.VISIBLE,ListStoreColumns.LOCAL,ListStoreColumns.INFEASIBLE], [
	cl.location.get_normal_form().print(true),
	lbt.code,
	lbt.displayName2,
	display,
	!hidden.filter(function (v) { return v == lbt.code; }).length,
	isLocal,
	turnedOff ? true : hideLocal ? isLocal : false	
      ]);
    });
    clocks_inhibitor = false;
  };
  worldClocksUpdate();
  clocksSettings.connect('changed::world-clocks', worldClocksUpdate);

  let infeasibleState = function () {
    let ls = ui.get_object('location-list');
    let turnedOff = !settings.get_boolean('in-calendar');
    let hideLocal = settings.get_boolean('hide-local');
    ls.foreach(function (store, path, iter) {
      store.set_value(iter, ListStoreColumns.INFEASIBLE,
		      turnedOff ? true : hideLocal ? store.get_value(iter, ListStoreColumns.LOCAL) : false);
    });
  };
  clock.connect('notify::timezone', worldClocksUpdate);
  settings.connect('changed::hide-local', infeasibleState);
  settings.connect('changed::in-calendar', infeasibleState);

  settings.connect('changed::hidden', function () {
    let ls = ui.get_object('location-list');
    let hidden = settings.get_value('hidden').deep_unpack();
    ls.foreach(function (store, path, iter) {
      let code = store.get_value(iter, ListStoreColumns.CODE);
      store.set_value(iter, ListStoreColumns.VISIBLE,
		      !hidden.filter(function (v) { return v == code; }).length);
    });
  });

  let positions_button_syms = ['LL', 'L1', 'LR', 'ML', 'M1', 'MR', 'RL', 'R9', 'RR'];
  positions_button_syms.map(function (pos_symbol) {
    let cl = pos_symbol;
    ui.get_object('position-' + cl).connect('toggled', function(object) {
      let cl2 = cl;
      if (position_inhibitor) { return; }
      position_inhibitor = true;
      positions_button_syms.map(function (j) {
  	let j2 = j;
  	ui.get_object('position-' + j2).set_active(j2 == cl2);
      });
      settings.set_string('button-position', cl2);
      position_inhibitor = false;
    });
  });
  let positionFromSetting = function() {
    ui.get_object('position-' + settings.get_string('button-position')).toggled();
  };
  positionFromSetting();
  settings.connect('changed::button-position', positionFromSetting);
  settings.connect('changed::name-map', worldClocksUpdate);
  
  let gladeSig = new GladeSignalHandlers(ui, world);
  ui.connect_signals_full(Lang.bind(gladeSig, gladeSig._connector));
  
  win.connect('destroy', function () {
    if (!openWinsCounter) {
      settings.run_dispose();
      settings = null;
      clocksSettings.run_dispose();
      clocksSettings = null;
      clock.run_dispose();
      clock = null;
    }
    else {
      openWinsCounter = openWinsCounter - 1;
    }
  });
  
  win.show();
  return win;
}
